function chart_pos_order_salesman (widget) {
    "use strict";

    var model = openerp;

    widget.ChartPosOrderSalesmanWidget = widget.Base.extend({
        template: 'ChartPosOrderSalesman',
        data: [],
        countPosOrder: [],
        resPartner: [],
        modelId: [],

        events: {
            // 'click canvas': 'showCustomers'
        },
        init: function (parent) {
            this._super(parent, {
                width: 6,
                height: 4
            });
        },
        start: function () {
            var self = this;
            self.fetchInitial();
        },
        fetchInitial:function() {
            var self = this;
            self.$el.block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#FAFAFA'
                }
            });

            self.$el.find('.widget-content.widget-loading').css('display','flex');

            self.fecthPosOrder().then(function (PosOrder) {
                return PosOrder;
            }).then(function (PosOrder) {
                self.PosOrder = PosOrder;
                return self.fecthResUser();
            }).then(function (ResUser) {
                self.ResUser = ResUser;
                return self.fetchProductRanking();
            });
        },

        // Usuario Logeado
        fetchCurrentUser: function() {
            var self = this;
            var ResUser = new model.web.Model('res.users');
            return ResUser.call('get_user', {
                context: new model.web.CompoundContext()
            });
        },

        // Lista de Graficos disponibles para el usuario
        fetchResUser: function(id) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name','chart_ids'];
            var domain = [['id','=',id]];
            var ResUser = new model.web.Model('res.users');
            ResUser.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },

        // Obtener detalles de la lista de graficos
        fetchChartList: function(chart_ids) {
            var self = this;
            var defer = $.Deferred();
            var fields = ['id','name'];
            var domain = [['id','in',chart_ids]];
            var ChartList = new model.web.Model('chart.list');
            ChartList.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });
            return defer;
        },

        // Obtener Pedidos
        fecthPosOrder: function() {
            var self = this;
            var desde = moment().format('YYYY-MM-01 00:00:00');
            var hasta = moment().add(1,'months').format('YYYY-MM-01 00:00:00');
            var defer = $.Deferred();
            var fields = ['id','date_order','partner_id','amount_total','user_id'];
            var domain = [['state', 'in', ['paid','done','invoiced']], ['partner_id', '!=', false], ['date_order', '>=', desde], ['date_order', '<', hasta]];
            var PosOrder = new model.web.Model('pos.order');
            PosOrder.query(fields).filter(domain).all().then(function (results) {
                defer.resolve(results);
            });

            return defer;
        },
        // Consultar Clientes
        fecthResUser: function() {
            var self = this;
            var defer = $.Deferred();
            var user_ids = _.map(self.PosOrder,function(map){
                return map.user_id[0];
            });
            var fields = ['id', 'name'];
            var domain = [['id', 'in', user_ids]];
            var ResUsers = new model.web.Model('res.users');
            ResUsers.query(fields).filter(domain).all().then(function(results) {
                defer.resolve(results);
            });

            return defer;
        },
        // Obtener facturas por cada clientes
        getPosOrder:function(partner_id) {
            var self = this;
            return _.flatten(_.filter(self.PosOrder,function (inv) {
                return inv.user_id[0] === partner_id;
            }));
        },
        // Generar el Ranking
        fetchProductRanking: function() {
            var self = this;
            var itemPartner;
            var itemPosOrder;
            var countPosOrder = 0
            var ranking = [];

            for (var i = 0; i < self.ResUser.length; i++) {
                itemPartner = self.ResUser[i];
                itemPosOrder = self.getPosOrder(itemPartner.id);

                var quantity = _.reduce(_.map(itemPosOrder,function(item) {
                    return item.amount_total;
                }),function(memo, num) {
                    return memo + num;
                },0);

                ranking.push({
                    id: itemPartner.id,
                    name: itemPartner.name,
                    countPosOrder: quantity
                });
            }
            ranking.sort(function (a, b) {
                return b.countPosOrder - a.countPosOrder
            });
            self.ranking = ranking;

            self.$el.unblock();
            self.$el.find('.widget-content.widget-loading').css('display','none');
            self.$el.find('.widget-title').find('span').text("Ventas por Vendedor");
            self.fetchChart(ranking);
        },
        // Generar Grafico
        fetchChart: function (ranking) {
            var self = this;
            var label = [];
            var body = [];
            var item;
            var rank = 5;

            if (ranking.length < rank && ranking.length > 0 )
                rank= ranking.length;

            for (var i = 0; i < rank; i++) {
                if (ranking[i]) {
                    item = ranking[i];
                }
                if (ranking.length === 0) {
                    item = {};
                    item.name = "N/A";
                    item.countPosOrder = 0;
                }
                label.push(item.name);
                body.push(item.countPosOrder);
            }
            // console.log(body);
            // Global method for setting Y axis number format.
            Chart.scaleService.updateScaleDefaults('linear', {
              ticks: {
                callback: function(tick) {
                    return tick.toLocaleString('de-DE');
                }
              }
            });
            Chart.defaults.global.tooltips.callbacks.label = function(tooltipItem, data) {
                var dataset = data.datasets[tooltipItem.datasetIndex];
                var datasetLabel = dataset.label || '';
                return datasetLabel +  dataset.data[tooltipItem.index].toLocaleString('de-DE');
            };
            var chart = new Chart(this.$el.find(".widget-content").find('canvas'), {
                type: 'horizontalBar',
                data: {
                    labels: label,
                    datasets: [
                        {
                            label: 'Ventas  ',
                            backgroundColor: '#e3f2fd',
                            borderColor: '#64b5f6',
                            hoverBackgroundColor: '#c8e6c9',
                            hoverBorderColor: '#66bb6a',
                            borderWidth: 2,
                            data: body,
                        }
                    ]
                },
                options: {
                    responsive: true,
                    // title: {
                    //     display: false,
                    // },
                    hover: {
                        mode: 'nearest',
                        intersect: true
                    },
                   // scales: {
                   //     xAxes: [{ stacked: true }],
                   //     yAxes: [{ stacked: true }],
                   // },
                   // legend: {
                   //     display: false,
                   // },
                    layout: {
                        padding: {
                            top: 20,
                            bottom: 0,
                            left : 0,
                            rigth: 0,
                        }
                    },
                }
            });
        },
        // llamar a la lista
        // showCustomers: function (e) {
        //     var self = this;
        //     if (self.ranking.length === 0 ) {
        //         model.web.notification.do_warn("Atención","Sin datos");
        //         return
        //     }
        //     var hoy = moment().format('YYYY-MM-DD');
        //     var partner_id = _.map(self.ranking,function(map){
        //         return map.id;
        //     })
        //     this.do_action({
        //         name: "Listado de clientes con más compras",
        //         type: 'ir.actions.act_window',
        //         res_model: "res.partner",
        //         views: [[false,'list'],[self.modelId[1],'form']],
        //         target: 'current',
        //         domain: [['id', 'in', partner_id]],
        //         context: {},
        //     });
        // }
    });
}
